@@ -1,3 +1,6 @@
+#' @keywords internal
+"_PACKAGE"
+
 ## usethis namespace: start
 #' @importFrom lifecycle deprecated
 #' @importFrom Rcpp sourceCpp
