@@ -1,9 +1,6 @@
 #' @include mesh.R
 #' @include deprecated.R
 
-#' @importFrom sp coordinates proj4string proj4string<-
-
-
 # fm_as_segm ####
 
 #' @export
@@ -12,7 +9,13 @@
 #' @param closed logical; whether to treat a point sequence as a closed polygon.
 #' Default: `FALSE`
 fm_as_segm.matrix <-
-  function(x, reverse = FALSE, grp = NULL, is.bnd = FALSE, crs = NULL, closed = FALSE, ...) {
+  function(x,
+           reverse = FALSE,
+           grp = NULL,
+           is.bnd = FALSE,
+           crs = NULL,
+           closed = FALSE,
+           ...) {
     loc <- x
     n <- dim(loc)[1L]
     if (closed) {
@@ -27,7 +30,7 @@ fm_as_segm.matrix <-
       }
     }
     fm_segm(
-      loc = loc, idx = idx, grp = grp, is.bnd = is.bnd, crs = fm_CRS(crs)
+      loc = loc, idx = idx, grp = grp, is.bnd = is.bnd, crs = fm_crs(crs)
     )
   }
 
@@ -36,6 +39,7 @@ fm_as_segm.matrix <-
 #' @rdname fm_as_segm
 fm_as_segm.SpatialPoints <-
   function(x, reverse = FALSE, grp = NULL, is.bnd = TRUE, closed = FALSE, ...) {
+    fm_safe_sp(force = TRUE)
     crs <- fm_CRS(x)
     loc <- sp::coordinates(x)
 
@@ -80,7 +84,7 @@ fm_as_segm.Line <-
       idx <- seq_len(n)
     }
     fm_segm(
-      loc = loc, idx = idx, grp = grp, is.bnd = FALSE, crs = fm_CRS(crs)
+      loc = loc, idx = idx, grp = grp, is.bnd = FALSE, crs = fm_crs(crs)
     )
   }
 
@@ -166,7 +170,7 @@ fm_as_segm.Polygons <-
   function(x, join = TRUE, crs = NULL, grp = NULL, ...) {
     segm <- as.list(lapply(
       x@Polygons,
-      function(x) fm_as_segm(x, crs = fm_CRS(crs), ...)
+      function(x) fm_as_segm(x, crs = fm_crs(crs), ...)
     ))
     if (join) {
       if (missing(grp)) {
@@ -198,7 +202,7 @@ fm_as_segm.Polygon <-
       loc = loc,
       idx = idx,
       is.bnd = TRUE,
-      crs = fm_CRS(crs)
+      crs = fm_crs(crs)
     )
   }
 
@@ -207,8 +211,8 @@ fm_as_segm.Polygon <-
 
 
 #' @export
-#' @describeIn fmesher-deprecated `r lifecycle::badge("deprecated")` in favour of
-#' [fm_as_segm()]
+#' @describeIn fmesher-deprecated `r lifecycle::badge("deprecated")` in favour
+#'   of [fm_as_segm()]
 fm_sp2segment <- function(...) {
   fm_as_segm(...)
 }
