@@ -22,6 +22,7 @@
 
 #include "fmesher_debuglog.h"
 #include "mesh.h"
+#include "mesh3.h"
 #include "trees.h"
 #include "vector.h"
 
@@ -184,6 +185,31 @@ public:
                                   TriangleLocator &locator);
 };
 
+class TetraLocator {
+
+public:
+  typedef BBoxLocator<double> bbox_locator_type;
+
+private:
+  const Mesh3 *mesh_;     /*! The mesh to be searched */
+  std::vector<int> dim_; /*! The order of the search dimensions, at most 3 */
+  bbox_type bbox_;       /*! Bounding boxes */
+  bbox_locator_type bbox_locator_; /*! The bbox searcher object */
+
+public:
+  TetraLocator(const Mesh3 *mesh, const std::vector<int> &dimensions,
+               bool use_interval_tree = true);
+
+  ~TetraLocator();
+
+  int locate(const Point &s, Double4 &b) const;
+
+  std::ostream &print(std::ostream &output);
+
+  friend std::ostream &operator<<(std::ostream &output,
+                                  TetraLocator &locator);
+};
+
 } /* namespace fmesh */
 
 #include "locator_t.h"
