@@ -51,3 +51,37 @@ test_that("Basic 1d FEM works", {
     expect_setequal(names(fem), names_fem[[configs$degree[k] + 1]])
   }
 })
+
+
+test_that("Basic fm_collect FEM works", {
+  mesh1 <- fmexample$mesh
+  fm_crs(mesh1) <- fm_crs("longlat_globe")
+  # Two transformations that both have kilometres as units:
+  mesh1 <- fm_transform(mesh1, fm_crs("mollweide_globe"))
+  mesh2 <- fm_transform(mesh1, fm_crs("globe"))
+  mesh3 <- fm_rcdt_2d(globe = 1, crs = fm_crs("globe"))
+
+  expect_error(
+    {
+      fm_collect(list(mesh1, mesh2, mesh3))
+    },
+    "All function spaces in a collection need to be of the same manifold type"
+  )
+  expect_no_error({
+    mesh <- fm_collect(list(mesh2, mesh3))
+  })
+
+  loc_sf <- fmexample$loc_sf
+  sf::st_crs(loc_sf) <- fm_crs("longlat_globe")
+  basis <- fm_basis(
+    mesh,
+    list(
+      c(loc_sf, loc_sf),
+      rep(1:2, each = NROW(loc_sf))
+    )
+  )
+
+  fem <- fm_fem(mesh, order = 4)
+  names_fem <- c("cc", "c0", "c1", "g1", "g2", "g3", "g4", "ta", "va")
+  expect_setequal(names(fem), names_fem)
+})
